@@ -25,6 +25,13 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyLong;
+import static org.mockito.ArgumentMatchers.eq;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
 
 import io.netty.buffer.ByteBuf;
 
@@ -50,6 +57,7 @@ import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieAccessor;
 import org.apache.bookkeeper.bookie.BookieException;
 import org.apache.bookkeeper.bookie.IndexPersistenceMgr;
+import org.apache.bookkeeper.client.AsyncCallback;
 import org.apache.bookkeeper.client.AsyncCallback.AddCallback;
 import org.apache.bookkeeper.client.BKException;
 import org.apache.bookkeeper.client.BookKeeper;
@@ -75,6 +83,7 @@ import org.apache.zookeeper.KeeperException;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
+import org.mockito.Mockito;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -365,6 +374,51 @@ public class AuditorPeriodicCheckTest extends BookKeeperClusterTestCase {
         }
     }
 
+    @Test
+    public void testGetLedgerFromZookeeperThrottled() throws Exception {
+        final int numberLedgers = 30;
+
+        // write ledgers into bookkeeper cluster
+        try {
+            for (AuditorElector e : auditorElectors.values()) {
+                e.shutdown();
+            }
+
+            for (int i = 0; i < numberLedgers; ++i) {
+                LedgerHandle lh = bkc.createLedger(3, 3, DigestType.CRC32, "passwd".getBytes());
+                for (int j = 0; j < 5; j++) {
+                    lh.addEntry("testdata".getBytes());
+                }
+                lh.close();
+            }
+        } catch (InterruptedException | BKException e) {
+            LOG.error("Failed to shutdown auditor elector or write data to ledgers ", e);
+            fail();
+        }
+
+        // create auditor and call `checkAllLedgers`
+        ServerConfiguration configuration = bsConfs.get(0);
+        configuration.setAuditorMaxNumberOfConcurrentOpenLedgerOperations(10);
+
+        Auditor auditor1 = new Auditor(Bookie.getBookieId(configuration).toString(),
+            configuration, NullStatsLogger.INSTANCE);
+        Auditor auditor = Mockito.spy(auditor1);
+
+        BookKeeper bookKeeper = Mockito.spy(auditor.getBookKeeper(configuration));
+        BookKeeperAdmin admin = Mockito.spy(auditor.getBookKeeperAdmin(bookKeeper));
+        when(auditor.getBookKeeper(configuration)).thenReturn(bookKeeper);
+        when(auditor.getBookKeeperAdmin(bookKeeper)).thenReturn(admin);
+
+        try {
+            auditor.checkAllLedgers();
+            verify(admin, times(numberLedgers)).asyncOpenLedgerNoRecovery(anyLong(),
+                any(AsyncCallback.OpenCallback.class), eq(null));
+        } catch (Exception e) {
+            LOG.error("Caught exception while checking all ledgers ", e);
+            fail();
+        }
+    }
+
     @Test
     public void testInitialDelayOfCheckAllLedgers() throws Exception {
         for (AuditorElector e : auditorElectors.values()) {
